#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define DHT11_PIN 7
uint8_t c=0,I_RH,D_RH,I_Temp,D_Temp,CheckSum;

volatile uint8_t prev_temp;

void play_annoying_alarm(int times)
{
	int i;
	
	for (i = 0; i < times; i++) {
		PORTB &= ~(1 << PB0);
		_delay_ms(50);
		PORTB |= (1 << PB0);
		_delay_ms(50);
	}
}

void initDHT(void)
{
    /* Set LED as output */
    //DDRA |= (1 << PA7);
    
    /* According to the DHT11's datasheet, the sensor needs about
       1-2 seconds to get ready when first getting power, so we
       wait
     */
    _delay_ms(1000);
}

void Timer1_init()
{
	// set up timer with prescaler = 1024 and CTC mode
    TCCR1B |= (1 << WGM12) | (5 << CS10);
  
    // initialize compare value
    OCR1A = 15625;
	
	/* enable overflow interrupt */
	TIMSK1 = (1 << OCIE1A);
}

void Request()				/* Microcontroller send start pulse/request */
{
	DDRA |= (1<<DHT11_PIN);
	PORTA &= ~(1<<DHT11_PIN);	/* set to low pin */
	_delay_ms(20);			/* wait for 20ms */
	PORTA |= (1<<DHT11_PIN);	/* set to high pin */
}

void Response()				/* receive response from DHT11 */
{
	DDRA &= ~(1<<DHT11_PIN);
	while(PINA & (1<<DHT11_PIN));
	while((PINA & (1<<DHT11_PIN))==0);
	while(PINA & (1<<DHT11_PIN));
}

uint8_t Receive_data()			/* receive data */
{	
	for (int q=0; q<8; q++)
	{
		while((PINA & (1<<DHT11_PIN)) == 0);  /* check received bit 0 or 1 */
		_delay_us(30);
		if(PINA & (1<<DHT11_PIN))/* if high pulse is greater than 30ms */
		c = (c<<1)|(0x01);	/* then its logic HIGH */
		else			/* otherwise its logic LOW */
		c = (c<<1);
		while(PINA & (1<<DHT11_PIN));
	}
	return c;
}

ISR(TIMER1_COMPA_vect) {
	Request();		/* send start pulse */
		Response();		/* receive response */
		I_RH=Receive_data();	/* store first eight bit in I_RH */
		D_RH=Receive_data();	/* store next eight bit in D_RH */
		I_Temp=Receive_data();	/* store next eight bit in I_Temp */
		D_Temp=Receive_data();	/* store next eight bit in D_Temp */
		CheckSum=Receive_data();/* store next eight bit in CheckSum */
		
		if (prev_temp < I_Temp) {
			play_annoying_alarm(1);
		} else if (prev_temp > I_Temp) {
			play_annoying_alarm(2);
		}
		
		
		if ((I_RH + D_RH + I_Temp + D_Temp) != CheckSum)
		{
			/* white */
			PORTA |= (7 << PA0);
		}
		
		else if (I_Temp <= 25)
		{	
			/* blue */
			PORTA &= ~(7 << PA0);
			PORTA |= (1 << PA2);
			
			/* if the mug gets colder than 25 degrees, start alarm */
			if (prev_temp > 25) {
				play_annoying_alarm(10);
			}
			
			prev_temp = I_Temp;
		}
		
		else if (I_Temp > 25 && I_Temp <= 30)
		{	
			/* green */
			PORTA &= ~(7 << PA0);
			PORTA |= (1 << PA1);
			
			prev_temp = I_Temp;
		}
		
		else if (I_Temp > 30 && I_Temp <= 35)
		{	
			/* red */
			PORTA &= ~(7 << PA0);
			PORTA |= (1 << PA0);
			
			prev_temp = I_Temp;
		}
		
		else if (I_Temp > 35 && I_Temp <= 40)
		{	
			/* purple */
			PORTA &= ~(7 << PA0);
			PORTA |= (1 << PA0) | (1 << PA2);
			
			prev_temp = I_Temp;
		}
		
		else if (I_Temp > 40)
		{	
			PORTA &= ~(7 << PA0);
			PORTA |= (1 << PA0) && (1 << PA1);
			
			prev_temp = I_Temp;
		}

}

int main(void)
{
	char data[5];
	initDHT();
	prev_temp = 0;

	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	
	DDRB |= (1 << PB0);
	PORTB |= (1 << PB0);
	
	Timer1_init();
	sei();
	
	while(1) {
	}
	
	return 0;
}